@@ -5,7 +5,7 @@ use strict;
 use warnings;
 use base qw( Exporter );
 
-our $VERSION = '0.14';
+our $VERSION = '0.16';
 
 use AnyEvent::Stomper::Frame;
 use AnyEvent::Stomper::Error;
@@ -67,19 +67,25 @@ sub new {
 
   my $self = bless {}, $class;
 
-  $self->{host} = $params{host} || D_HOST;
-  $self->{port} = $params{port} || D_PORT;
-  $self->{login}    = $params{login};
-  $self->{passcode} = $params{passcode};
-  $self->{vhost}    = $params{vhost};
+  $self->{host}            = $params{host} || D_HOST;
+  $self->{port}            = $params{port} || D_PORT;
+  $self->{login}           = $params{login};
+  $self->{passcode}        = $params{passcode};
+  $self->{vhost}           = $params{vhost};
+  $self->{lazy}            = $params{lazy};
+  $self->{handle_params}   = $params{handle_params} || {};
+  $self->{default_headers} = $params{default_headers} || {};
+  $self->{on_connect}      = $params{on_connect};
+  $self->{on_disconnect}   = $params{on_disconnect};
 
   if ( defined $params{heartbeat} ) {
     unless ( ref( $params{heartbeat} ) eq 'ARRAY' ) {
-      croak qq{"heartbeat" must be specified as array reference};
+      croak q{"heartbeat" must be specified as array reference};
     }
+
     foreach my $val ( @{ $params{heartbeat} } ) {
       if ( $val =~ /\D/ ) {
-        croak qq{"heartbeat" values must be an integer numbers};
+        croak q{"heartbeat" values must be an integer numbers};
       }
     }
 
@@ -89,10 +95,17 @@ sub new {
     $self->{heartbeat} = D_HEARTBEAT;
   }
 
-  $self->{lazy}          = $params{lazy};
-  $self->{handle_params} = $params{handle_params} || {};
-  $self->{on_connect}    = $params{on_connect};
-  $self->{on_disconnect} = $params{on_disconnect};
+  if ( defined $params{command_headers} ) {
+    unless ( ref( $params{command_headers} ) eq 'HASH' ) {
+      croak q{"command_headers" must be specified as hash reference};
+    }
+
+    my %command_headers;
+    while ( my ( $cmd_name, $headers ) = each %{ $params{command_headers} } ) {
+      $command_headers{ uc($cmd_name) } = $headers;
+    }
+    $self->{command_headers} = \%command_headers;
+  }
 
   $self->connection_timeout( $params{connection_timeout} );
   $self->reconnect_interval( $params{reconnect_interval} );
@@ -121,14 +134,6 @@ sub execute {
   return;
 }
 
-sub force_disconnect {
-  my $self = shift;
-
-  $self->_disconnect();
-
-  return;
-}
-
 # Generate methods
 {
   no strict qw( refs );
@@ -210,6 +215,14 @@ sub on_error {
   }
 }
 
+sub force_disconnect {
+  my $self = shift;
+
+  $self->_disconnect();
+
+  return;
+}
+
 sub _connect {
   my $self = shift;
 
@@ -337,9 +350,9 @@ sub _create_on_read {
       return if $handle->destroyed;
 
       if ( defined $cmd_name ) {
-        my $content_length = $headers->{'content-length'};
-
-        if ( defined $content_length ) {
+        my $content_length;
+        if ( defined $headers->{'content-length'} ) {
+          $content_length = $headers->{'content-length'};
           return if length( $handle->{rbuf} ) < $content_length + 1;
         }
         else {
@@ -390,18 +403,28 @@ sub _prepare {
       $cbs{on_receipt} = pop @{$args};
     }
   }
-  my %cmd_headers = @{$args};
-  if ( defined $cmd_headers{on_receipt} ) {
-    $cbs{on_receipt} = delete $cmd_headers{on_receipt};
+
+  my %headers = @{$args};
+  foreach my $name ( qw( on_receipt on_message ) ) {
+    if ( defined $headers{$name} ) {
+      $cbs{$name} = delete $headers{$name};
+    }
   }
-  if ( defined $cmd_headers{on_message} ) {
-    $cbs{on_message} = delete $cmd_headers{on_message};
-  }
-  my $body = delete $cmd_headers{body};
+  my $body = delete $headers{body};
+
+  %headers = (
+    %{ $self->{default_headers} },
+
+    defined $self->{command_headers}{$cmd_name}
+    ? %{ $self->{command_headers}{$cmd_name} }
+    : (),
+
+    %headers,
+  );
 
   my $cmd = {
     name    => $cmd_name,
-    headers => \%cmd_headers,
+    headers => \%headers,
     body    => $body,
     %cbs,
   };
@@ -414,7 +437,7 @@ sub _prepare {
       my $err     = shift;
 
       if ( defined $err ) {
-        $self->{on_error}->( $err, $receipt );
+        $self->{on_error}->($err);
         return;
       }
     };
@@ -480,39 +503,43 @@ sub _push_write {
   my $self = shift;
   my $cmd  = shift;
 
-  my $cmd_headers = $cmd->{headers};
+  my $headers = $cmd->{headers};
 
   my $need_receipt;
   if ( exists $NEED_RECEIPT{ $cmd->{name} }
-    || defined $cmd_headers->{receipt} )
+    || defined $headers->{receipt} )
   {
     $need_receipt = 1;
     if ( $cmd->{name} eq 'CONNECT' ) {
       $self->{_pending_receipts}{CONNECTED} = $cmd;
     }
     else {
-      if ( !defined $cmd_headers->{receipt}
-        || $cmd_headers->{receipt} eq 'auto' )
+      if ( !defined $headers->{receipt}
+        || $headers->{receipt} eq 'auto' )
       {
-        $cmd_headers->{receipt} = $self->{_session_id} . '@@'
+        $headers->{receipt} = $self->{_session_id} . '@@'
             . $self->{_receipt_seq}++;
       }
-      $self->{_pending_receipts}{ $cmd_headers->{receipt} } = $cmd;
+      $self->{_pending_receipts}{ $headers->{receipt} } = $cmd;
     }
   }
 
-  unless ( defined $cmd->{body} ) {
-    $cmd->{body} = '';
+  my $body = $cmd->{body};
+  unless ( defined $body ) {
+    $body = '';
   }
-  unless ( defined $cmd_headers->{'content-length'} ) {
-    $cmd_headers->{'content-length'} = length( $cmd->{body} );
+  unless ( defined $headers->{'content-length'} ) {
+    $headers->{'content-length'} = length($body);
   }
 
-  my $frame_str = uc( $cmd->{name} ) . EOL;
-  while ( my ( $name, $value ) = each %{$cmd_headers} ) {
+  my $frame_str = $cmd->{name} . EOL;
+  while ( my ( $name, $value ) = each %{$headers} ) {
+    unless ( defined $value ) {
+      $value = '';
+    }
     $frame_str .= _escape($name) . ':' . _escape($value) . EOL;
   }
-  $frame_str .= EOL . $cmd->{body} . "\0";
+  $frame_str .= EOL . "$body\0";
 
   $self->{_handle}->push_write($frame_str);
 
@@ -688,8 +715,8 @@ sub _process_receipt {
   }
 
   if ( exists $SUBUNSUB_CMDS{ $cmd->{name} } ) {
-    my $cmd_headers = $cmd->{headers};
-    my $sub_id = $cmd_headers->{id} || $cmd_headers->{destination};
+    my $headers = $cmd->{headers};
+    my $sub_id  = $headers->{id} || $headers->{destination};
 
     if ( $cmd->{name} eq 'SUBSCRIBE' ) {
       $self->{_subs}{$sub_id} = $cmd;
@@ -1006,8 +1033,16 @@ Disabled by default.
 
 =item reconnect_interval => $reconnect_interval
 
-If the parameter is specified, the client will try to reconnect only after
-this interval. Commands executed between performance will be queued.
+If the connection to the server was lost, the client will try to restore the
+connection when you execute next command. By default reconnection is performed
+immediately, on next command execution. If the C<reconnect_interval> parameter
+is specified, the client will try to reconnect only after this interval and
+commands executed between reconnections will be queued.
+
+The client will try to reconnect only once and, if attempt fails, the error
+object is passed to command callback. If you need several attempts of the
+reconnection, you must retry a command from the callback as many times, as you
+need.
 
   reconnect_interval => 5,
 
@@ -1025,6 +1060,30 @@ Specifies L<AnyEvent::Handle> parameters.
 Enabling of the C<autocork> parameter can improve performance. See
 documentation on L<AnyEvent::Handle> for more information.
 
+=item default_headers => \%headers
+
+Specifies default headers for all outgoing frames.
+
+  default_headers => {
+    'x-foo' => 'foo_value',
+    'x-bar' => 'bar_value',
+  }
+
+=item command_headers
+
+Specifies default headers for particular commands.
+
+  command_headers => {
+    SEND => {
+      receipt => 'auto',
+    },
+
+    SUBSCRIBE => {
+      durable => 'true',
+      ack     => 'client',
+    },
+  }
+
 =item on_connect => $cb->()
 
 The C<on_connect> callback is called when the connection is successfully
@@ -1060,7 +1119,7 @@ way to specify the command callback.
 
 If you want to receive C<RECEIPT> frame, you must specify C<receipt> header.
 The C<receipt> header can take the special value C<auto>. If it set, the
-receipt identificator will be generated automatically by the client. The
+receipt identifier will be generated automatically by the client. The
 C<RECEIPT> frame is passed to the command callback in first argument as the
 object of the class L<AnyEvent::Stomper::Frame>. If the C<receipt> header is
 not specified the first argument of the command callback will be C<undef>.
@@ -1069,10 +1128,11 @@ For commands C<SUBSCRIBE>, C<UNSUBSCRIBE>, C<DISCONNECT> the client
 automatically adds C<receipt> header for internal usage.
 
 The command callback is called in one of two cases depending on the presence of
-the C<receipt> header: when the command was successfully sent to the server or
-when the C<RECEIPT> frame will be received. If any error occurred during the
-command execution, the error object is passed to the callback in second
-argument. Error object is the instance of the class L<AnyEvent::Stomper::Error>.
+the C<receipt> header. First case, when the command was successfully sent to
+the server. Second case, when the C<RECEIPT> frame will be received. If any
+error occurred during the command execution, the error object is passed to the
+callback in second argument. Error object is the instance of the class
+L<AnyEvent::Stomper::Error>.
 
 The command callback is optional. If it is not specified and any error
 occurred, the C<on_error> callback of the client is called.
@@ -1392,14 +1452,14 @@ Gets or sets the C<on_error> callback.
 The method for forced disconnection. All uncompleted operations will be
 aborted.
 
-=head1 CONNECTION POOL
+=head1 WORKING WITH CLUSTER
 
-If you have the cluster or set of STOMP servers, you can use
-L<AnyEvent::Stomper::Pool> to work with them.
+If you have the cluster of STOMP servers, you can use
+L<AnyEvent::Stomper::Cluster> to work with it.
 
 =head1 SEE ALSO
 
-L<AnyEvent::Stomper::Pool>
+L<AnyEvent::Stomper::Cluster>
 
 =head1 AUTHOR
 
@@ -1409,7 +1469,7 @@ Sponsored by SMS Online, E<lt>dev.opensource@sms-online.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2016, Eugene Ponizovsky, SMS Online. All rights reserved.
+Copyright (c) 2016-2017, Eugene Ponizovsky, SMS Online. All rights reserved.
 
 This module is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
