@@ -5,7 +5,7 @@ use strict;
 use warnings;
 use base qw( Exporter );
 
-our $VERSION = '0.33_01';
+our $VERSION = '0.16';
 
 use AnyEvent::Stomper;
 use AnyEvent::Stomper::Error;
@@ -23,17 +23,6 @@ BEGIN {
 
 use constant \%ERROR_CODES;
 
-my %ACK_CMDS = (
-  ACK  => 1,
-  NACK => 1,
-);
-
-my %CAN_REPEAT = (
-  SEND      => 1,
-  SUBSCRIBE => 1,
-  BEGIN     => 1,
-);
-
 
 sub new {
   my $class  = shift;
@@ -100,6 +89,14 @@ sub execute {
   }
 }
 
+sub get_node {
+  my $self = shift;
+  my $host = shift;
+  my $port = shift;
+
+  return $self->{_nodes_pool}{"$host:$port"};
+}
+
 sub nodes {
   my $self = shift;
   return values %{ $self->{_nodes_pool} };
@@ -128,7 +125,7 @@ sub on_error {
 sub force_disconnect {
   my $self = shift;
 
-  foreach my $node ( $self->nodes ) {
+  foreach my $node ( values %{ $self->{_nodes_pool} } ) {
     $node->force_disconnect;
   }
   $self->_reset_internals;
@@ -229,34 +226,31 @@ sub _prepare {
   my $cmd_name = uc(shift);
   my $args     = shift;
 
-  my %params;
-
+  my %cbs;
   if ( ref( $args->[-1] ) eq 'CODE'
     && scalar @{$args} % 2 > 0 )
   {
     if ( $cmd_name eq 'SUBSCRIBE' ) {
-      $params{on_message} = pop @{$args};
+      $cbs{on_message} = pop @{$args};
     }
     else {
-      $params{on_receipt} = pop @{$args};
+      $cbs{on_receipt} = pop @{$args};
     }
   }
 
   my %headers = @{$args};
-
-  foreach my $name ( qw( body on_receipt on_message on_node_error ) ) {
+  foreach my $name ( qw( on_receipt on_message on_node_error ) ) {
     if ( defined $headers{$name} ) {
-      $params{$name} = delete $headers{$name};
+      $cbs{$name} = delete $headers{$name};
     }
   }
-  if ( exists $ACK_CMDS{$cmd_name} ) {
-    $params{message} = delete $headers{message};
-  }
+  my $body = delete $headers{body};
 
   my $cmd = {
     name    => $cmd_name,
     headers => \%headers,
-    %params,
+    body    => $body,
+    %cbs,
   };
 
   unless ( defined $cmd->{on_receipt} ) {
@@ -295,6 +289,7 @@ sub _execute {
 
       if ( defined $err ) {
         my $err_code = $err->code;
+        $fails_cnt++;
 
         my $on_node_error = $cmd->{on_node_error} || $self->{on_node_error};
         if ( defined $on_node_error ) {
@@ -302,10 +297,9 @@ sub _execute {
           $on_node_error->( $err, $node->host, $node->port );
         }
 
-        if ( $CAN_REPEAT{ $cmd->{name} }
-          && $err_code != E_OPRN_ERROR
+        if ( $err_code != E_OPRN_ERROR
           && $err_code != E_CONN_CLOSED_BY_CLIENT
-          && ++$fails_cnt < scalar @{ $self->{_nodes} } )
+          && $fails_cnt < scalar @{ $self->{_nodes} } )
         {
           $self->_execute( $cmd, $fails_cnt );
           return;
@@ -319,10 +313,6 @@ sub _execute {
       $cmd->{on_receipt}->($receipt);
     },
 
-    defined $cmd->{message}
-    ? ( message => $cmd->{message} )
-    : (),
-
     defined $cmd->{on_message}
     ? ( on_message => $cmd->{on_message} )
     : (),
@@ -383,30 +373,31 @@ AnyEvent::Stomper::Cluster - The client for the cluster of STOMP servers
     id          => 'foo',
     destination => '/queue/foo',
 
-    on_receipt => sub {
-      my $err = $_[1];
+    { on_receipt => sub {
+        my $err = $_[1];
+
+        if ( defined $err ) {
+          warn $err->message . "\n";
+          $cv->send;
+
+          return;
+        }
+
+        $cluster->send(
+          destination => '/queue/foo',
+          body        => 'Hello, world!',
+        );
+      },
+
+      on_message => sub {
+        my $msg = shift;
+
+        my $body = $msg->body;
+        print "Consumed: $body\n";
 
-      if ( defined $err ) {
-        warn $err->message . "\n";
         $cv->send;
-
-        return;
-      }
-
-      $cluster->send(
-        destination => '/queue/foo',
-        body        => 'Hello, world!',
-      );
-    },
-
-    on_message => sub {
-      my $msg = shift;
-
-      my $body = $msg->body;
-      print "Consumed: $body\n";
-
-      $cv->send;
-    },
+      },
+    }
   );
 
   $cv->recv;
@@ -759,7 +750,7 @@ callback, this callback will be act as C<on_message> callback.
       my $body    = $msg->body;
 
       # message handling...
-    },
+    }
   );
 
 =head2 unsubscribe( [ %params ] [, $cb->( $receipt, $err ) ] )
@@ -791,14 +782,12 @@ The method is used to remove an existing subscription.
 The method is used to acknowledge consumption of a message from a subscription
 using C<client> or C<client-individual> acknowledgment. Any messages received
 from such a subscription will not be considered to have been consumed until the
-message has been acknowledged via an C<ack()> method. Method C<ack()> must be
-called with required parameter C<message> in which must be specified the
-C<MESSAGE> frame.
+message has been acknowledged via an C<ack()> method.
 
-  $stomper->ack( message => $msg );
+  $cluster->ack( id => $ack_id );
 
-  $stomper->ack(
-    message => $msg,
+  $cluster->ack(
+    id      => $ack_id,
     receipt => 'auto',
 
     sub {
@@ -820,14 +809,12 @@ C<MESSAGE> frame.
 =head2 nack( [ %params ] [, $cb->( $receipt, $err ) ] )
 
 The C<nack> method is the opposite of C<ack> method. It is used to tell the
-server that the client did not consume the message. Method C<nack()> must be
-called with required parameter C<message> in which must be specified the
-C<MESSAGE> frame.
+server that the client did not consume the message.
 
-  $stomper->nack( message => $msg );
+  $cluster->nack( id => $ack_id );
 
-  $stomper->nack(
-    message => $msg,
+  $cluster->nack(
+    id      => $ack_id,
     receipt => 'auto',
 
     sub {
@@ -854,7 +841,7 @@ The method C<begin> is used to start a transaction.
 
 The method C<commit> is used to commit a transaction.
 
-=head2 abort( [ %params ] [, $cb->( $receipt, $err ) ] )
+=head2 abort([ %params ] [, $cb->( $receipt, $err ) ] )
 
 The method C<abort> is used to roll back a transaction.
 
@@ -906,6 +893,10 @@ Full list of error codes see in documentation on L<AnyEvent::Stomper>.
 
 =head1 OTHER METHODS
 
+=head2 get_node( $host, $port )
+
+Gets node by host and port.
+
 =head2 nodes()
 
 Gets all available nodes.
